// Copyright (c) 2012-2017 Algo Communication Products Ltd.

import QtQuick 1.1

Item {
	id: button
	property alias frame: frame
	property alias pic: pic
	property alias label: label
	property alias highlight: highlight
	property int anim: 5
	property int label_res_top: 5
	property int label_res_height: 30

	property int transparent: 0

	property bool enable: true
	property string button_bg: ""

	property int text_padding: 10

	Rectangle {
		id: frame
		anchors { fill: parent;}
		border.color: "black"
		border.width: 1
	}

	Image {
		id: pic
		y: 0
		fillMode: Image.PreserveAspectFit
		anchors.horizontalCenter: button.horizontalCenter
		height: button.height
	}

	Image {
		id: highlight
		y: 0
		height: pic.height
		width: pic.width
		opacity: 0
	}

	Text {
		id: invisibleText
		x: parent.x + text_padding
		y: parent.y
		width: parent.width - 2*text_padding
		wrapMode : Text.Wrap
		text: label.text
		font.pixelSize: parent.height * 0.25
		horizontalAlignment : Text.AlignHCenter
		font.bold: true
		visible: false
	}

	Text {
		id: label
		x: parent.x + text_padding
		y: parent.y
		width: parent.width - 2*text_padding
		wrapMode : Text.Wrap
		horizontalAlignment : Text.AlignHCenter

		font.bold: true
		smooth: true
		//font.pixelSize: parent.height * 0.25
		font.pixelSize: {
			if ( invisibleText.paintedHeight > 0 &&
				parent.height > 0 &&
				invisibleText.paintedHeight > parent.height ) {
					var scaled_size = Math.round( 0.1 + 0.9 * parent.height * 0.25 * parent.height / invisibleText.paintedHeight );
					return scaled_size;
			} else if ( invisibleText.paintedWidth > 0 &&
				parent.width > 0 &&
				invisibleText.paintedWidth > parent.width ) {
					var scaled_size = Math.round( 0.1 + 0.9 * parent.height * 0.25 * parent.width / invisibleText.paintedWidth );
					return scaled_size;
			} else {
				var normal_size = Math.round(0.1+0.9 * parent.height * 0.25) ;
				return normal_size;
			}
		}
	}

        MouseArea {
		anchors { fill: parent;}

		onPressed: {
			if (!enable)
				return;

			if (highlight.source != "")
				highlight.opacity = 0.5;
		}

		onReleased: {
			if (!enable)
				return;

			highlight.opacity = 0 ;
		}

		onClicked: {
			if (!enable) {
				main.handleWakeup();
				return;
			}

			if (button.click)
				button.click();
		}
	}

	Component.onCompleted: {
		if (pic.source == "") {
			//pic.height = 0;
			//label.anchors.centerIn = button;

			if(transparent == 0) {
				if(button_bg == "")
					pic.source = main.theme + "button-bg.png"
				else
					pic.source = button_bg;
			}

			label.anchors.centerIn = button;
			frame.opacity  = 0;
		} else {
			if(transparent == 1) {
				makeTransparent();
			} else {
				label.anchors.horizontalCenter = button.horizontalCenter;
				label.anchors.top = pic.bottom;
				label.anchors.topMargin = label_res_top;
			}
		}
	}

	function makeTransparent()
	{
		frame.opacity = 0
		pic.opacity = 0
		label.opacity = 0
	}

}
