// Copyright (c) 2012-2017 Algo Communication Products Ltd.

import QtQuick 1.1
import algo.boat 1.0
import "sail_directory_page.js" as SailDirectoryPageJs

Item {
	property int seq: 0
	property string page_id: ""

	property string selected_name: ""
	property string selected_address: ""
	property string selected_phone: ""

	property string directory_name:"";
	property string directory_description:"";

	property int selected_index: -1 ;

	property int phonebook_current_start_index: 0;

	property string highlight_colour: "#E0E0E0";

	property int show_contact_icon: 1;
	property int show_contact_address: 1;
	property int show_details_when_selected: 1;

	property string search_input: ""
	property int display_text_limit: 14

	property int dir_change: 0

	function reset()
	{
	}


	function search_append(ch)
	{
		if( search_input.length +1 > display_text_limit ) {
			return;
		}

		display.text = display.text + ch;
		search_input = search_input + ch;
	}

	function search_clear()
	{
		display.text = "Search";
		search_input = "";
	}


	function hide_keyboard_only()
	{
		keyboardPage.opacity = 0;
	}

	function show_keyboard_only()
	{
		keyboardPage.opacity = 1;
	}


	property string return_page: "";

	width: 640; height: 480

	function reset_addressbook()
	{
		if (dir_change == 0) {
			dir_change = 1;
		}

		search_clear();
		addrModel.updateAddress("");

		dir.positionViewAtIndex(0, ListView.Begining);
		selected_index  = -1;
		dir.currentIndex = selected_index;

		dir_change = 0;
	}

	Image {
		source: main.theme + "directory-bg.png"
		anchors.centerIn: parent
	}

	MouseArea {
		anchors { fill: parent; }
		onPressed: {
			main.handleWakeup();
			hide_keyboard_only();
		}
	}



	BoatAddrModel {
		id: addrModel;
	}

	// The delegate for each section header
	// We don't have section headers now
	Component {
		id: sectionHeading
		Rectangle {
			width: 400; height: 20
			color: highlight_colour;
			Text {
				text: section
				anchors.right:parent.right
				anchors.rightMargin:20
				font.bold: true
			}
		}
	}

	Rectangle {
		id: dir_area
		opacity: 1

		x: 0; y: 75

		width: 400; height: 400
		color: "white"

		ListView {
			id: dir
			model: addrModel
			anchors.fill: parent
			cacheBuffer: 50*64
			snapMode: ListView.SnapToItem
			delegate: SailContactDelegate {
				onClicked: {
					main.handleTouch();
					hide_keyboard_only();
				}
			}


			highlight: Rectangle {
				color: highlight_colour
			}
			highlightMoveSpeed: 9999999
			highlightFollowsCurrentItem: true
			clip: true
			interactive: false
			flickDeceleration:1000

			//section.property: "group"
			//section.criteria: ViewSection.FullString
			//section.delegate: sectionHeading

			onMovementStarted:
			{
				page_num.opacity = 0;
			}

			onMovementEnded:
			{
				main.handleTouch();
				page_num.opacity = 1;
			}
		}

	}


	//Call button
	SailButton {
		x: 455; y: 15; width: 125 ; height: 84
		frame.border.width: 0
		frame.color: "transparent"
		pic.y: 0
		pic.source: main.theme + "call-125x84.png"
		highlight.source: main.theme + "white-125x84.png"

		label_res_top: 0
		label_res_height: 0

		opacity: {
			if (dir.currentIndex < 0)
				return 0;
			else
				return 1;
		}

		function click()
		{
			boatSystem.logd("call " + selected_phone);
			phonePage.callOut(selected_phone);

			reset_addressbook();
			hide_keyboard_only();
		}
	}


	//up-arrow
	Image {
		id: page_up_button
		source: main.theme + "up.png"
		x:455
		y:139

		width:125
		height:100

		opacity: {
			if( dir.currentIndex == 0 )
				return 0.5
			else
				return 	1;
		}


		MouseArea {
			anchors { fill: parent; }

			onPressed: {
				main.handleTouch();

				if( dir_change == 0 ) {
					dir_change = 1;
				}

				//if we are already at top, can't go up any more
				if( dir.currentIndex == 0 ) {
					dir_change = 0;
					return;
				}

				if( dir.currentIndex < 0 )
						dir.currentIndex = 0;

				else if ((dir.currentIndex - 6) >= 0)
						dir.currentIndex = dir.currentIndex - 6;
				else
						dir.currentIndex = 0;

				dir.positionViewAtIndex(dir.currentIndex, ListView.Center);
				dir_change = 0;
			}
		}
	}

	//down-arrow
	Image {
		id: page_down_button
		source: main.theme + "down.png"
		x: 455
		y: 241

		width:125
		height:100

		opacity: {
			if (dir.currentIndex == dir.count - 1)
				return 0.5;
			else
				return 	1;
		}

		MouseArea {
			anchors { fill: parent; }

			onPressed: {
				main.handleTouch();

				if( dir_change == 0 ) {
					dir_change = 1;
				}

				//if we are already at bottom, can't go down any more
				if( dir.currentIndex == dir.count - 1) {
					dir_change = 0;
					return;
				}

				if( dir.currentIndex < 0 )
						dir.currentIndex = Math.floor(6/2);
				else if ((dir.currentIndex + 6) >= dir.count)
						dir.currentIndex = dir.count - 1;
				else
						dir.currentIndex = dir.currentIndex + 6;

				dir.positionViewAtIndex(dir.currentIndex, ListView.Center);
				dir_change = 0;
			}
		}
	}

	Text {
		id: page_num

		x: 510
		y: 351

		text: {
			var temp_index = (dir.currentIndex < 0 ) ? 1 : dir.currentIndex

			var num1 = Math.floor(temp_index / 7)  + 1;
			num1 = num1 > 0 ? num1 : 1;

			var num2 = Math.ceil(dir.count /7);
			num2 = num2 > 0 ? num2 : 1;

			if(num1 > num2)
				num1 = num2;

			return num1 + " / " + num2;
		}

		font.pointSize: 12; font.bold: true
		color:"black"
	}


	//go back to previous page
	Item {
		width: 84 ; height: 84
		x: 425
		y: 381

		Image {
			id: picMintoToMain; anchors.fill: parent; smooth: true
			source: main.theme + "back.png"
		}

		MouseArea {
			anchors { fill: parent; }
			onPressed: {
				main.handleTouch();
				main.switchPageBack();
				reset_addressbook();
				hide_keyboard_only();
			}
		}
	}


	//go back to home page
	Item {
		width: 84 ; height: 84
		x: 530
		y: 381

		opacity:1

		Image {
			id: picExitMinto; anchors.fill: parent; smooth: true
			source: main.theme + "home.png"
		}

		MouseArea {
			anchors { fill: parent; }
			onPressed: {
				main.handleTouch();
				main.switchPageHome();
				reset_addressbook();
				hide_keyboard_only();
			}
		}
	}


	SailKeyboardPage {
		id: keyboardPage
		opacity: 0

		x:0
		y:250
		width: 640
		height: 280


		onClicked: {
			main.handleTouch();
			search_append(keyboardPage.text);
			addrModel.updateAddress( search_input );
	        }

	}

	//Search Text box

	Image {
		x: 5 ; y: 5; width: 320; height: 65
		source: main.theme + "text-box.png"
		opacity: 1

		Text {
			id: display
			width: parent.width - 10
			height: parent.height
			text:"";
			font.pointSize: 22; color: "#000000"
			font.bold : true
			horizontalAlignment: Text.AlignHCenter
			verticalAlignment: Text.AlignVCenter
		}

		Image {
			x: 0 ; y: 0; width: 320; height: 65
			id: search_image
			source: main.theme + "search-text.png"
			opacity: {
					if (display.text=="Search")
						return 1;
					else
						return 0;
			}
		}

		MouseArea {
			anchors { fill: parent; }
		        onPressed: {
				main.handleTouch();
				keyboardPage.opacity  = 1;
				reset_addressbook();
				display.text = "";
				search_input = "";
		        }
		}
	}

//cancel serach
	Image {
		x: 320; y: 5
		width: 65; height: 65

		source: main.theme + "x-big.png"
		opacity: 1

		MouseArea {
			anchors { fill: parent; }
		        onPressed: {
				main.handleTouch();
				keyboardPage.opacity = 0;

				display.text = "Search";
				search_input = "";
				reset_addressbook(); 
		        }
		}
	}


	Component.onCompleted: {
		SailDirectoryPageJs.config();
		reset_addressbook();
	}

	Component.onDestruction: {
		// do nothing
	}
}
