// Copyright (c) 2012-2017 Algo Communication Products Ltd.

import QtQuick 1.1

Item {

	property string page_id: ""

	property int call_status: 0
	property int hangup_mode: 0
	property string end_call_target: ""

	width: 640; height: 480

	MouseArea {
		anchors { fill: parent; }
	}

	Image {
		id: phone_bg
		source: main.theme + "call-progress-bg.png"
		anchors.centerIn: parent
	}

	Image {
		x: 120; y: 110
		source: main.theme + "call-established.png"
	}


	//this is intentionally set to invisible
	//in case we need to display custom text, we can set 'opacity' to 1
	Text {
		id: party
		x: 170; y: 170; width: 350
		text: ""
		wrapMode: Text.WrapAtWordBoundaryOrAnywhere
		font.pointSize: 16; font.bold: true
		color:"black"
		opacity: 0
	}

	Item {
		id: hangup_button
		x: 490; y: 380
		width: 130; height: 80

		opacity: 1

		Image {
			anchors.fill: parent;
			source: main.theme + "call-hangup.png"
		}

		MouseArea {
			anchors { fill: parent; }
			onPressed: {
				if (hangup_mode == 1)
					hangup(1);
			}
		}
	}

	function setEndCallTarget(target)
	{
		end_call_target = target;
	}

	function setStatus(msg)
	{
		boatSystem.logd("PhonePage setStatus: " + msg);

		hangup_button.opacity = hangup_mode;

		switch (msg) {
		case "Call active":
			phone_bg.source = main.theme +
				"call-established-bg.png";
			party.text = "Call Established";
			break;

		case "Idle":
			phone_bg.source = main.theme +
				"call-progress-bg.png";
			party.text = msg;
			break;

		default:
			phone_bg.source = main.theme +
				"call-progress-bg.png";
			party.text = msg;
			break;
		}
	}

	function callOut(number)
	{
		boatSystem.sendPhoneMsg("call start " + number);

		if (call_status == 0)
			main.switchPage("phone");

		call_status = 1;
	}

	function callIn(number)
	{
		if (call_status == 0) {
			main.switchPage("phone");
		}

		call_status = 1;
	}

	function handle_video_stopped()
	{
		boatSystem.logd("PhonePage handle_video_stopped: do nothing");
	}

	function hangup(local)
	{
		if (call_status == 0) {
			boatSystem.logd("PhonePage hangup: Already hungup");
			return;
		}
		call_status = 0;

		if (local) {
			main.handleTouch();
			boatSystem.sendPhoneMsg("call stop");
		}

		setStatus("");
		if (end_call_target == "back") {
			main.switchPageBack();
		} else {
			main.switchPageHome();
		}
	}
}
