// Copyright (c) 2012-2017 Algo Communication Products Ltd.

import QtQuick 1.1

Item {
	property string page_id: ""

	width: 640; height: 480
	property int max_slideshow_time: 0

	Image {
		id: display_image

		anchors.centerIn: parent
		opacity: 0
		cache: false

		MouseArea {
			anchors { fill: parent; }

			onPressed: {
				main.handleTouch();
			}
		}
	}

	Text {
		id: message
		x: 0
		y: 240
		width: 640
		wrapMode : Text.Wrap
		horizontalAlignment : Text.AlignHCenter
		verticalAlignment : Text.AlignVCenter

		text: "No slide found"
		opacity: 0

		font.pixelSize: 28
		font.bold: true
	}

	Timer {
		id: interval_timer
		interval: 5000
		running: false
		repeat: true

		onTriggered: {
			setSlide(boatSlideShow.getNextSlide());
		}
	}

	Timer {
		id: timeout_timer
		interval: 60000
		running: false;
		repeat: false;

		onTriggered: {
			stop();
			main.switchPageBack();
			main.handleTouch();
		}
	}

	function setInterval(interval)
	{
		if (interval < 0 || interval > 300)
			interval = 5;
		boatSystem.logd("slideshow interval set to " + interval +
			"seconds");
		interval_timer.interval = interval * 1000;
	}

	function setTimeout(interval)
	{
		if (interval < 0)
			interval = 1800;
		boatSystem.logd("slideshow timeout set to " + interval +
			"seconds");
		timeout_timer.interval = interval * 1000;
	}

	function setSlide(text)
	{
		boatSystem.logd("slideshow use slide " + text );
		if (text == "") {
			message.opacity = 1;
			display_image.source = main.theme + "dp-bg.png";
		} else {
			display_image.source = text;
			message.opacity = 0;
		}
	}

	function start()
	{
		boatSystem.logd("start slideshow");
		boatSlideShow.startSlideShow();
		setSlide(boatSlideShow.getNextSlide());
		display_image.opacity = 1;
		interval_timer.running = true;
	}

	function stop()
	{
		boatSystem.logd("stop slideshow");
		interval_timer.running = false;
		boatSlideShow.endSlideShow();
		display_image.opacity = 0;
	}

	function startTimeout()
	{
		if (timeout_timer.interval == 0 ) {
			boatSystem.logd("slideshow timeout timer is disabled");
			return;
		}
		timeout_timer.running = 1;
	}
}
