// Copyright (c) 2012-2017 Algo Communication Products Ltd.

Qt.include("sail_config.js")

var pages = {};
var history = [];
var cur = null;
var home = null;

var lcd_def;
var lcd_cur;
var lcd_idle;
var pres_state = 0;

var pres_use;
var pres_action;
var pres_dest;
var idle_resume;

function createPages()
{
	var button_page_comp = Qt.createComponent("SailButtonPage.qml");
	var directory_page_comp = Qt.createComponent("SailDirectoryPage.qml");
	var npage = configGetInt("ui.page.active");

	if (npage == 0) {
		var welcome_page_comp = Qt.createComponent("welcome.qml");
		var welcomePage = welcome_page_comp.createObject(main);
		pages['welcome'] = welcomePage;
		pages['phone'] = phonePage;
		phonePage.setEndCallTarget('back');
		home = pages['welcome'];
		cur = home;
		pres_use = configGetInt("ui.pres.use");
		boatMedia.setClickSound("/apps/uiapp/rc/sounds/click.wav");
		return;
	}

	for (var i = 1; i <= npage; i++) {
		var page_template = configGetPage(i, "general.tmpl");
		var page_name = "page" + i;
		var page;
		if (page_template == "button") {
			page = button_page_comp.createObject(main, {seq : i});
		} else if (page_template == "directory") {
			page = directory_page_comp.createObject(main, {seq : i});
		} else {
			boatSystem.logd("unknown template:", page_template);
			continue;
		}
		page.opacity = 0;
		pages[page_name] = page;
	}

	var door_page_comp = Qt.createComponent("SailAccessPage.qml");

	pages['access1'] = door_page_comp.createObject(main,
		{page_id : "access1", door_id : "0"});
	pages['access1'].setAccessCodes(boatSystem.getDoorAccessCode(1));

	pages['access2'] = door_page_comp.createObject(main,
		{page_id : "access2", door_id : "1"});
	pages['access2'].setAccessCodes(boatSystem.getDoorAccessCode(2));

	pages['access3'] = door_page_comp.createObject(main,
		{page_id : "access3", door_id : "2"});
	pages['access3'].setAccessCodes(boatSystem.getDoorAccessCode(3));

	pages['access4'] = door_page_comp.createObject(main,
		{page_id : "access4", door_id : "3"});
	pages['access4'].setAccessCodes(boatSystem.getDoorAccessCode(4));

	pages['phone'] = phonePage;
	pages['dialer'] = dialerPage;
	pages['lock'] = lockPage;
	pages['auth'] = authPage;
	pages['status'] = statusPage;
	pages['vlb'] = vlbPage;
	pages['slideshow'] = slideshowPage;
	pages['blank'] = blankPage;

	var home_page = configGet("ui.page.home");
	if (home_page != "")
		home = pages[home_page];
	else
		home = pages["page1"];

	cur = home;
	cur.opacity = 1;

	boatSystem.logi("theme dir: " + main.theme);

	var click = "user/sounds/click.wav";
	if (!boatSystem.testFile(main.appdir + click))
		click = "rc/sounds/click.wav";
	boatMedia.setClickSound(main.appdir + click);
	boatSystem.logi("click tone: " + click);

	var slide_interval = configGetInt("ui.slide.interval");
	if (slide_interval <= 0 || slide_interval > 300)
		slide_interval = 5;
	slideshowPage.setInterval(slide_interval);
	slideshowPage.setTimeout(configGetInt("ui.slide.timeout"));

	pres_use = configGetInt("ui.pres.use");
	pres_action = configGet("ui.pres.action");
	pres_dest = configGet("ui.pres.dest");
	idle_resume = configGet("ui.idle.resume");

	lcd_idle = configGetInt("ui.lcd.idle");
	if (lcd_idle < 0 || lcd_idle > 10)
		lcd_idle = 0;

	lcd_def = configGetInt("ui.lcd.brt");
	if (lcd_def <= 0 || lcd_def > 10)
		lcd_def = 5;

	lcd_cur = lcd_def;

	var hangup_mode = configGetInt("ui.call.hangup");
	phonePage.hangup_mode = hangup_mode;
	phonePage.setStatus("");
	var end_call_target = configGet("ui.call.end");
	phonePage.setEndCallTarget(end_call_target);

	boatSystem.setBrightness(lcd_def);

	var idle_timeout = configGetInt("ui.idle.timeout");
	idleTimer.interval = idle_timeout * 1000;
	if (idle_timeout > 0)
		idleTimer.start();
}

function handleCmd(cmd)
{
	var args;

	boatSystem.logd("qml cmd: " + cmd);
	args = cmd.split(" ");
	if (args[0] == "call") {
		if (args[1] == "start") {
			handleCallStart();
		} else if (args[1] == "stop") {
			video_call_flag = 0;
			if (checkCurPage("lock"))
				switchPageBack();
			phonePage.hangup(0);
			if (idleTimer.interval > 0)
				idleTimer.restart();
		} else if (args[1] == "connect") {
		} else if (args[1] == "update") {
			var call_status = boatSystem.getCallStatus();
			phonePage.setStatus(call_status);
		} else if (args[1] == "error") {
			phonePage.setStatus("Device is not ready for call");
			phonePage.hangup(0);
			if (idleTimer.interval > 0)
				idleTimer.restart();
		}

	} else if (args[0] == "pres") {
		if (args[1] == "start") {
			handlePres(1);
		} else if (args[1] == "stop") {
			handlePres(0);
		}
	} else if (args[0] == "door") {
		if (args[1] == "unlock") {
			handleDoor(args[2], 1);
		} else if (args[1] == "invalid") {
			handleDoor(args[2], -1);
		} else if (args[1] == "lock") {
			handleDoor(args[2], 0);
		}
	} else if (args[0] == "video") {
		if (args[1] == "start") {
			video_call_flag = 1;
		} else if (args[1] == "stop") {
			phonePage.handle_video_stopped();
			video_call_flag = 0;
		}
	} else if (args[0] == "page") {
		if (args[1] == "show") {
			switchPage(args[2]);
			if (idleTimer.interval > 0)
				idleTimer.restart();
		} else if (args[1] == "refresh") {
			refreshPage(args[2]);
			if (idleTimer.interval > 0)
				idleTimer.restart();
		}
	}
}

function handleIdleTimerTriggered()
{
	if (phonePage.call_status == 1) {
		if (idleTimer.interval > 0)
			idleTimer.restart();
		return;
	}

	for (var entry in pages) {
		if (pages[entry].reset) {
			pages[entry].reset();
		}
	}

	if (lcd_idle > 0 && lcd_cur == lcd_def &&
		pres_state == 0 ) {
		boatSystem.setBrightness(lcd_idle);
		lcd_cur = lcd_idle;
	}

	if (pres_state == 1) {
		boatSystem.logd("visitor present, ignoring any timeout event");
		if (idleTimer.interval > 0)
			idleTimer.restart();
		return;
	}

	var idle_action = configGet("ui.idle.action");
	if (idle_action == "slideshow")	{
		boatSystem.logd("idle timeout: going to slideshow");
		// start maximum slideshow timer
		slideshowPage.startTimeout();
		switchPage("slideshow");
	} else if (idle_action == "home") {
		if(cur == slideshowPage) {
			slideshowPage.stop();
		}
		switchPageHome();
	} else if (idle_action == "blank") {
		boatSystem.setBrightness(0);
		lcd_cur = 0;
		switchPage("blank");
	} else {
	}
}

function switchPage(dest)
{
	var size = 0;

	if(dest == "" || dest.length <= 0) {
		boatSystem.logd("switchPage: bad destination");
		return;
	}

	if(dest == "back") {
		switchPageBack();
		return;
	} else if (dest == "home") {
		switchPageHome();
		return;
	}

	if (cur == pages[dest])
		return;

	if (pages[dest]) {
		cur.opacity = 0;
		size = history.push(cur.page_id);
		if (size > 10)
			history.shift();
		cur = pages[dest];
		cur.opacity = 1;

		if (cur.page_id)
			boatSystem.logd("switchPage: " + cur.page_id);

		if (dest == 'slideshow') {
			slideshowPage.start();
		}
	} else {
		boatSystem.logd("switchPage: unknown destination, " + dest);
		return;
	}
}

function refreshPage(page_name)
{
	if (page_name.substring(0, 4) !== "page")
		return;

	var ishome = (home == pages[page_name]);
	var iscur = (cur == pages[page_name]);

	var id = parseInt(page_name.substring(4));
	var npage = configGetInt("ui.page.active");
	if (id <= 0 && id > npage)
		return;

	if (idleTimer.interval > 0)
		idleTimer.restart();

	if (pages[page_name]) {
		pages[page_name].destroy();
		gc();
	}

	var page_template = configGetPage(id, "general.tmpl");
	var comp;
	var page;
	if (page_template == "button") {
		comp = Qt.createComponent("SailButtonPage.qml");
		page = comp.createObject(main, {seq : id});
	} else if (page_template == "directory") {
		comp = Qt.createComponent("SailDirectoryPage.qml");
		page = comp.createObject(main, {seq : id});
	} else {
		boatSystem.logd("unknown template:", page_template);
	}

	page.opacity = 0;
	pages[page_name] = page;

	if (ishome)
		home = page;

	if (iscur) {
		page.opacity = 1;
		cur = page;
	} else {
		page.opacity = 0;
	}
}

function checkCurPage(id)
{
	return (cur.page_id == id);
}

function switchPageBack()
{
	var id = history.pop();

	cur.opacity = 0;
	cur = pages[id];
	if (! cur) {
		cur = home;
	}
	cur.opacity = 1;

	boatSystem.logd("switchPageBack: current page, " + cur.page_id);
}

function switchPageHome()
{
	if (cur == home)
		return;

	cur.opacity = 0;
	history = [];
	cur = home;
	cur.opacity = 1;

	boatSystem.logd("switchPageHome");
}

function handlePres(state)
{
	if (!pres_use)
		return;

	if (pres_state == state)
		return;

	boatSystem.logd("pres state = " + pres_state);
	pres_state = state;

	if (phonePage.call_status == 1)
		return;

	if (cur == pages['welcome']) {
		if (state == 1)
			boatMedia.playClickSound();

		pages['welcome'].showPres(state);
		return;
	}

	if (idleTimer.interval > 0)
		idleTimer.restart();

	if (state == 1) {
		// restore LCD brightness
		if (lcd_cur != lcd_def) {
			boatSystem.setBrightness(lcd_def);
			lcd_cur = lcd_def;
		}

		if(cur == slideshowPage || cur == blankPage) {
			if (cur == slideshowPage)
				slideshowPage.stop();

			if (idle_resume == "back")
				switchPageBack();
			else if (idle_resume == "home")
				switchPageHome();

			if (idleTimer.interval > 0)
				idleTimer.restart();

			if (pres_action == "call") {
				boatSystem.logd("pres triggered call, ext:" +
					pres_dest);
				phonePage.callOut(pres_dest);
			}
		}
	}
}

function handleDoor(id, state)
{
	if (state == 1) {
		switchPage("lock");
		lockPage.showUnlocked();
		boatSystem.setDisplay("ui");
	} else if (state == -1) {
		switchPage("lock");
		lockPage.showInvalid();
		boatSystem.setDisplay("ui");
	} else {
		if (video_call_flag == 1) {
			boatSystem.setDisplay("video");
		}
		if(cur == lockPage) {
			switchPageBack();
		}
	}
}

function handleWakeup()
{
	if (lcd_cur != lcd_def) {
		boatSystem.setBrightness(lcd_def);
		lcd_cur = lcd_def;
	}

	if (idleTimer.interval > 0)
		idleTimer.restart();
}

function handleTouch()
{
	if (cur == slideshowPage || cur == blankPage) {
		if (cur == slideshowPage)
			slideshowPage.stop();

		if (idle_resume == "back")
			switchPageBack();
		else if (idle_resume == "home")
			switchPageHome();
	}

	boatMedia.playClickSound();
	handleWakeup();
}

function handleCallStart()
{
	if (cur == slideshowPage) {
		slideshowPage.stop();

		if (idle_resume == "back")
			switchPageBack();
		else if (idle_resume == "home")
			switchPageHome();
	}

	handleWakeup();
	phonePage.callIn("");
	idleTimer.stop();
}
