-- Copyright (c) 2016-2017 Algo Communication Products Ltd.

require('session')
require('webpage')

local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
end

--[[
icons: {
	folder: "folder",
	doc: "file-word-o",
	excel: "file-excel-o",
	pdf: "file-pdf-o",
	pp: "file-powerpoint-o",
	text: "file-text-o",
	video: "file-video-o",
	image: "file-image-o",
	code: "file-code-o",
	audio: "file-audio-o",
	archive: "file-archive-o",
	file: "file-o"
}
]]

dp = '/apps/uiapp/user/'
local ft

function add_file(id, entity, fn)
	local ext = fn:match("^.+(%..+)$")
	local ftype = 'file'
	if ext == '.png' or ext == '.jpg' or ext == '.jpeg' or
		ext == '.gif' then
		ftype = 'image'
	elseif ext == '.conf' then
		ftype = 'text'
	elseif ext == '.csv' then
		ftype = 'excel'
	end
	return string.format('{"id":"%s","value":"%s","type":"%s",\z
		"size":%d,"date":%d',
		id, entity, ftype, lfs.attributes(fn, 'size'),
		lfs.attributes(fn, 'modification'))
end

function add_dir(id, entity, fn)
	return string.format('{"id":"%s","value":"%s","type":"folder",\z
		"size":%d,"date":%d,"data":',
		id, entity, lfs.attributes(fn, 'size'),
		lfs.attributes(fn, 'modification'))
end

-- recursive function
function browse_folder(root)
	local first = true
	local dirlist = {}
	local filelist = {}
	local path, fullpath, id, fn

	if root == '' then
		path = ''
	else
		path = root .. '/'
	end
	fullpath = dp .. path
	ft = ft .. '['
	for node in lfs.dir(dp .. root) do
		if node ~= '.' and node ~= '..' then
			local mode = lfs.attributes(fullpath .. node, 'mode')
			if mode == 'directory' then
				table.insert(dirlist, node)
			elseif mode == 'file' then
				table.insert(filelist, node)
			end
		end
	end
	table.sort(dirlist)
	table.sort(filelist)

	for i, node in ipairs(dirlist) do
		if not first then
			ft = ft .. ','
		end
		id = path .. node
		fn = fullpath .. node
		ft = ft .. add_dir(id, node, fn)
		browse_folder(id)
		ft = ft .. '}'
		first = false
	end

	for i, node in ipairs(filelist) do
		if not first then
			ft = ft .. ','
		end
		id = path .. node
		fn = dp .. id
		ft = ft .. add_file(id, node, fn)
		ft = ft .. '}'
		first = false
	end

	ft = ft .. ']'
end

function send_filetree()
	ft = '[{"value":"Files","type":"folder",\z
		"size":0,"date":0,"id":"/","data":'
	browse_folder('')
	ft = ft .. ',"open":true}]'
	mg.write('HTTP/1.1 200 OK\r\nConnection: close\r\n\z
		Content-Type: application/json\r\n\r\n' ..
		ft)
end

function send_ok()
	mg.write('HTTP/1.1 200 OK\r\nConnection: close\r\n\z
		Content-Type: text/html\r\n\r\n["ok"]')
end

function string:split(sep)
	local sep, fields = sep or ":", {}
	local pattern = string.format("([^%s]+)", sep)
	self:gsub(pattern, function(c) fields[#fields+1] = c end)
	return fields
end

local method = mg.request_info.request_method
if method == 'GET' then
	send_filetree()
elseif method == 'POST' then
	local cl = mg.request_info.http_headers['Content-Length']
	local params = {}
	webpage_parse_post(cl, params)
	if params['action'] == 'files' then
		send_filetree()
	elseif params['action'] == 'remove' then
		cmd = 'cd ' .. dp .. '; rm -rf "' ..
			string.gsub(params['source'], ',', '" "') .. '"'
		os.execute(cmd)
		send_ok()
	elseif params['action'] == 'create' then
		if params['type'] == 'folder' then
			cmd = 'mkdir -p "' .. dp .. params['source'] .. '"'
			os.execute(cmd)
			send_ok()
		end
	elseif params['action'] == 'rename' then
		cmd = string.format('cd "%s"; mv "%s" $(dirname "%s")/"%s"',
			dp, params['source'], params['source'],
			params['target'])
		os.execute(cmd)
		send_ok()
	elseif params['action'] == 'move' then
		local files = string.split(params['source'], ',')
		local src = ''
		for i, fn in ipairs(files) do
			src = src .. '"' .. dp .. fn .. '" '
		end
		cmd = 'mv ' .. src .. ' "' .. dp .. params['target'] .. '"'
		os.execute(cmd)
		send_ok()
	elseif params['action'] == 'download' then
		session_download(params['source'])
		session_send_file(dp .. params['source'])
	end
end
