-- Copyright (c) 2015-2017 Algo Communication Products Ltd.

require('session')
require('webpage')
require('conf')
require('prod')
require('uicommon')

uicommon_load_conf()
local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end
session_save()

local flags = {['any']=0, ['main']=0}

local cl = mg.request_info.http_headers['Content-Length']
if cl then
	local settings = {}

	webpage_parse_post(cl, settings)

	conf_update('ui.page.active', settings, flags, 'main', 1)
	conf_update('ui.page.home', settings, flags, 'main', 1)
	conf_update('ui.lcd.brt', settings, flags, 'main', 1)
	conf_update('ui.door.unlockmsg', settings, flags, 'ui', 1)

	conf_update('ui.idle.timeout', settings, flags, 'main', 1)
	conf_update('ui.idle.action', settings, flags, 'main', 1)
	conf_update('ui.idle.resume', settings, flags, 'main', 1)
	conf_update('ui.lcd.idle', settings, flags, 'main', 1)

	conf_update('ui.pres.use', settings, flags, 'main', 1)
	conf_update('ui.pres.action', settings, flags, 'main', 1)
	conf_update('ui.pres.dest', settings, flags, 'main', 1)

	conf_update('ui.slide.interval', settings, flags, 'main', 1)

	conf_update('ui.call.hangup', settings, flags, 'main', 1)
	conf_update('ui.call.end', settings, flags, 'main', 1)

	conf_update('ui.page1.button.bg', settings, flags, 'main', 1)
	conf_update('ui.page1.general.tmpl', settings, flags, 'main', 1)

	if flags['any'] == 1 then
		conf_save()
	end
	if flags['main'] == 1 then
		os.execute('/etc/init.d/algo-uieng restart')
	end
end

local form_start = webpage_cst_formstart('/control/ui/index.lua')

local page_start = webpage_cst_pagestart('General Settings')

local js_show = [[
<script type="text/javascript">

function display_element(id, state)
{
	document.getElementById(id).style.display = state ? 'block' : 'none';
}

function read_radio_value(name)
{
	var radios = document.getElementsByName(name);
	for (var i = 0, length = radios.length; i < length; i++) {
		if (radios[i].checked) {
			return (radios[i].value);
		}
	}
	return '';
}

function read_select_value(name)
{
	var selects = document.getElementsByName(name)[0]
	var opt;
	for (var i = 0, len = selects.options.length; i < len; i++) {
		opt = selects.options[i];
		if (selects.options[i].selected === true)
			return selects.options[i].value;
	}
	return '';
}

function convert_number(value)
{
	if (isNaN(value))
		return 0;
	else
		return parseInt(value);
}

function show_page()
{
	var state;
	var action;

	state = (convert_number(read_select_value("ui.page.active")) > 0);
	display_element("home_page", state);
	state = (convert_number(read_select_value("ui.idle.timeout")) > 0);
	display_element('idle_action', state);
	display_element('idle_lcd', state);
	action = read_select_value("ui.idle.action");
	if (action == "" || action == "home")
		display_element('idle_resume', 0);
	else
		display_element('idle_resume', state);
	state = (read_radio_value("ui.pres.use") == "1");
	display_element('pres_action', state);
	if (state)
		state = (read_select_value("ui.pres.action") == "call");
	else
		state = false;
	display_element('pres_dest', state);
}

</script>]]

local show_page = 'show_page();'

local secthd_global = webpage_cst_secthd('Global Settings')

local page_desc = {}
local page_val = {}
table.insert(page_desc, 'None')
table.insert(page_val, '0')
for i = 1, 20 do
	table.insert(page_desc, tostring(i))
	table.insert(page_val, tostring(i))
end

local p_npage = webpage_cst_param('Number of Active Pages',
	webpage_cst_select('ui.page.active', page_desc, page_val, true,
		 nil, nil, nil, show_page),
	'Number of configurable pages that will be created as part of \z
	 the User Interface.')

page_desc = {}
page_val = {}
local npage = conf_getpn('ui.page.active')
-- always show page #1
if npage == 0 then
	npage = 1
end
for i = 1, npage do
	local desc = conf_getp('ui.page' .. i .. '.general.desc')

	if desc and desc ~= '' then
		table.insert(page_desc, 'Page ' .. i .. ' - ' .. desc)
	else
		table.insert(page_desc, 'Page ' .. i)
	end
	table.insert(page_val, 'page' .. i)
end
local p_start = webpage_cst_param('Home Page',
	webpage_cst_select('ui.page.home', page_desc, page_val),
	'User interface starts with this page.', 'home_page')

-- hidden parameters
local p_page1_tmpl = webpage_cst_param('Template',
	webpage_cst_text('ui.page1.general.tmpl'),
	nil, nil, 1)
local p_page1_bg = webpage_cst_param('Background',
	webpage_cst_text('ui.page1.button.bg'),
	nil, nil, 1)

local brt_desc = {'1 - Darkest', '2', '3', '4', '5',
	'6', '7', '8', '9', '10 - Brightest'}
local brt_level = {'1', '2', '3', '4', '5', '6', '7', '8', '9', '10'}
local p_brt = webpage_cst_param('Default LCD Brightness',
	webpage_cst_select('ui.lcd.brt', brt_desc, brt_level, true),
	'Screen backlight brightness.')

local p_unlock_msg = webpage_cst_param('Door Unlock Message',
	webpage_cst_enabled('ui.door.unlockmsg'),
	'Display a message when door is open using DTMF.')

local secthd_timeout = webpage_cst_secthd('Timeout Settings')

local p_timeout = webpage_cst_param('User Interface Timeout',
	webpage_cst_select('ui.idle.timeout',
		{'None', ' 5 seconds', '10 seconds', '20 seconds',
		 '30 seconds', '1 minute', '2 minutes', '5 minutes',
		 '10 minutes', '30 minutes', '1 hour'},
		{'0', '5', '10', '20', '30', '60', '120', '300',
		 '600', '1800', '3600'},
		true, nil, nil, nil, show_page),
	'The User Interface can dim or activate a slideshow after a period \z
	 of inactivity.')

local p_idle_action = webpage_cst_param('Timeout Action',
	webpage_cst_select('ui.idle.action',
		{'None', 'Go to Homepage', 'Activate Slideshow',
		 'Turn off LCD'},
		{'', 'home', 'slideshow', 'blank'},
		nil, nil, nil, nil, show_page),
	nil, 'idle_action', 1)

local p_idle_resume = webpage_cst_param('Action on Resume',
	webpage_cst_select('ui.idle.resume',
		{'Return to Previous Page', 'Go to Homepage'},
		{'back', 'home'}),
	nil, 'idle_resume', 1)

local p_idle_lcd = webpage_cst_param('LCD Brightness after Timeout Event',
	webpage_cst_select('ui.lcd.idle',
		{'&lt;Default&gt;',
		 '1', '2', '3', '4', '5', '6', '7', '8', '9', '10'},
		{'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10'},
		true),
	nil, 'idle_lcd', 1)

local secthd_pres = webpage_cst_secthd('Presence Detection Settings')

local p_pres = webpage_cst_param('Presence Sensor for User Interface',
	webpage_cst_enabled('ui.pres.use', {show_page, show_page}),
	'Detects when a visitor is present within 80cm directly in front of \z
	 the screen.  Will cancel the slideshow or screen dimming if \z
	 these features are active.')

local p_pres_action = webpage_cst_param('Action when Presence Detected',
	webpage_cst_select('ui.pres.action',
		{'None', 'Call'},
		{'', 'call'},
		nil, nil, nil, nil, show_page),
	'Automatically initiates a phone call when a visitor is detected.',
	'pres_action')

local p_pres_dest = webpage_cst_param('Extension to Dial',
	webpage_cst_text('ui.pres.dest'),
	nil, 'pres_dest', 1)

local secthd_slide = webpage_cst_secthd('Slideshow Settings', nil, nil,
	'A slideshow allows a series of images to be played in sequence.<br>\z
	 The slideshow can be activated after a period of inactivity \z
	 (using the Timeout settings above), or directly from a button \z
	 press.<br>\z
	 Use the <a href="/control/ui/fm.lua">File Manager</a> tab to upload \z
	 images to the "slides" folder. Images must be 640x480 resolution, \z
	 in PNG, JPG, or GIF format.<br>\z
	 The slideshow can be exited by a touchscreen press, \z
	 or by the presence detection sensor (see settings above).')

local p_slide_interval = webpage_cst_param('Interval between Slides (seconds)',
	webpage_cst_text('ui.slide.interval'),
	'Valid value is between 0 and 300 (5 minutes).')

local secthd_call = webpage_cst_secthd('Call Control Settings')

local p_hangup = webpage_cst_param('Hangup Button',
	webpage_cst_enabled('ui.call.hangup'),
	'Displays a "Hangup" button on the screen during a phone call \z
	 to allow the visitor to end this call')

local p_endcall = webpage_cst_param('Action at End of Call',
	webpage_cst_select('ui.call.end',
		{'Return to Previous Page', 'Go to Homepage'},
		{'back', 'home'}))

local js_showall = [[<script type="text/javascript">
show_page();
</script>]]

mg.write(prod_cst_header() ..
	uicommon_cst_menu('General') ..
	webpage_cst_mchd(flags['any'], 0) ..
	form_start .. page_start .. js_show ..
	secthd_global .. webpage_subsect_hd ..
	p_npage .. p_start .. p_page1_tmpl .. p_page1_bg ..
	p_brt .. p_unlock_msg ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_timeout .. webpage_subsect_hd ..
	p_timeout .. p_idle_action .. p_idle_resume .. p_idle_lcd ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_pres .. webpage_subsect_hd ..
	p_pres .. p_pres_action .. p_pres_dest ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_slide .. webpage_subsect_hd ..
	p_slide_interval ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_call .. webpage_subsect_hd ..
	p_hangup .. p_endcall ..
	webpage_subsect_ft .. webpage_sect_ft ..
	webpage_page_end .. webpage_form_end_save .. js_showall ..
	webpage_mc_ft ..
	webpage_footer)
