-- Copyright (c) 2015-2017 Algo Communication Products Ltd.

require('session')
require('webpage')
require('conf')
require('prod')
require('uicommon')

uicommon_load_conf()
local auth_failed = session_load()
if auth_failed == 1 then
	session_reset()
	return
else
	session_start_page()
end
session_save()

local page = mg.get_var(mg.request_info.query_string, 'page')
local pagepf = 'ui.page' .. page

local flags = {['any']=0, ['main']=0}

local cl = mg.request_info.http_headers['Content-Length']
if cl then
	local settings = {}

	webpage_parse_post(cl, settings)
	conf_update(pagepf .. '.general.desc', settings, flags, 'main', 1)
	conf_update(pagepf .. '.general.tmpl', settings, flags, 'main', 1)

	conf_update(pagepf .. '.button.bg', settings, flags, 'main', 1)
	conf_update(pagepf .. '.button.layout', settings, flags, 'main', 1)
	conf_update(pagepf .. '.button.back', settings, flags, 'main', 1)
	conf_update(pagepf .. '.button.home', settings, flags, 'main', 1)
	conf_update(pagepf .. '.button.action', settings, flags, 'main', 1)
	conf_update(pagepf .. '.button.dest', settings, flags, 'main', 1)
	conf_update(pagepf .. '.button.target', settings, flags, 'main', 1)

	local i
	for i = 1, 9 do
		conf_update(pagepf .. '.button' .. i .. '.use',
			settings, flags, 'main', 1)
		conf_update(pagepf .. '.button' .. i .. '.label',
			settings, flags, 'main', 1)
		conf_update(pagepf .. '.button' .. i .. '.action',
			settings, flags, 'main', 1)
		conf_update(pagepf .. '.button' .. i .. '.dest',
			settings, flags, 'main', 1)
		conf_update(pagepf .. '.button' .. i .. '.target',
			settings, flags, 'main', 1)
	end

	conf_update(pagepf .. '.dir.addrbook', settings, flags, 'main', 1)

	if flags['any'] == 1 then
		conf_save()
		os.execute('/opt/algo/bin/prod-sendcmd-uiapp "page refresh page'
			.. page .. '"')
	end
end

-- assign default values if uninitialized
if conf_getp(pagepf .. '.button.bg') == '' then
	conf_setp(pagepf .. '.button.bg',
		'rc/background/default1-sky-clouds.png', 1)
end

if conf_getp(pagepf .. '.button.target') == '' then
	conf_setp(pagepf .. '.button.target', 'home', 1)
end

for i = 1, 9 do
	if conf_getp(pagepf .. '.button' .. i .. '.target') == '' then
		conf_setp(pagepf .. '.button' .. i .. '.target', 'home', 1)
	end
end

if conf_getp(pagepf .. '.dir.addrbook') == '' then
	conf_setp(pagepf .. '.dir.addrbook', 'sample1.txt', 1)
end

local form_start = webpage_cst_formstart('/control/ui/page.lua?page=' .. page)

local page_start = webpage_cst_pagestart('Page Settings')

local secthd_general = webpage_cst_secthd("General")

local js_all = [[
<script type="text/javascript">
var page = ]] .. page .. [[;

function display_element(id, state)
{
	document.getElementById(id).style.display = state ? 'block' : 'none';
}

function control_template(tmpl)
{
	display_element('sect_button', tmpl == 'button');
	display_element('sect_directory', tmpl == 'directory');
}

function read_radio_value(name)
{
	var radios = document.getElementsByName(name);
	for (var i = 0, length = radios.length; i < length; i++) {
		if (radios[i].checked) {
			return (radios[i].value);
		}
	}
	return '';
}

function read_select_value(name) {
	var selects = document.getElementsByName(name)[0]
	var opt;
	for (var i = 0, len = selects.options.length; i < len; i++) {
		opt = selects.options[i];
		if (selects.options[i].selected === true)
			return selects.options[i].value;
	}
	return '';
}

function control_button(seq)
{
	var btn_use = read_radio_value('ui.page' + page +
		'.button' + seq + '.use');
	var action = read_radio_value('ui.page' + page +
		'.button' + seq + '.action');
	if (btn_use == '1') {
		display_element('btn' + seq + '_label', 1);
		display_element('btn' + seq + '_act', 1);
		display_element('btn' + seq + '_dest', action == 'call');
		display_element('btn' + seq + '_target', action == 'goto');
	} else {
		display_element('btn' + seq + '_label', 0);
		display_element('btn' + seq + '_act', 0);
		display_element('btn' + seq + '_dest', 0);
		display_element('btn' + seq + '_target', 0);
	}
}

function control_page_act()
{
	var action = read_radio_value('ui.page' + page + '.button.action');
	display_element('page_dest', action == 'call');
	display_element('page_target', action == 'goto');
}

var nbts = {'': 0, '1': 1, '2' :2, '3' :3, '2x2': 4,
	'3x2': 6, '3x3': 9, '3x3fs': 9};

// enable all visible buttons by default for new button layout
function update_buttons()
{
	var layout = read_select_value('ui.page' + page + '.button.layout');
	var nbt = nbts[layout];

	for (var i = 1; i <= nbt; i++) {
		document.getElementsByName(
			'ui.page' + page + '.button' + i + '.use')[0].checked
			= true;
	}
	for (var i = nbt + 1; i <= 9; i++) {
		document.getElementsByName(
			'ui.page' + page + '.button' + i + '.use')[1].checked
			= true;
	}
}

function display_svglist(name, state)
{
	var svglist = document.querySelectorAll('.' + name);
	var len = svglist.length;
	var display = state ? 'inline' : 'none';

	for (var i = 0; i < len; i++) {
		svglist[i].style.display = display;
	}
}

function display_loc(name)
{
	var svgall = ['svg2x2', 'svg3x2', 'svg3x3'];
	var len = svgall.length;

	for (var i = 0; i < len; i++) {
		display_svglist(svgall[i], (name == svgall[i]) ? 1 : 0);
	}
}

function show_page()
{
	var tmpl = read_radio_value('ui.page' + page + '.general.tmpl');
	control_template(tmpl);
	control_page_act();

	var layout = read_select_value('ui.page' + page + '.button.layout');
	var nav = (layout != '3x3fs');
	display_element('back_button', nav);
	display_element('home_button', nav);

	var nbt = nbts[layout];
	for (var i = 1; i <= nbt; i++) {
		display_element('subsect_button' + i, true);
		control_button(i);
	}
	for (var i = nbt + 1; i <= 9; i++) {
		display_element('subsect_button' + i, false);
	}
	if (nbt == 4) {
		display_loc('svg2x2');
	} else if (nbt == 6) {
		display_loc('svg3x2');
	} else if (nbt == 9) {
		display_loc('svg3x3');
	} else {
		display_loc('none');
	}
}
</script>]]

local show_page = 'show_page();'

local p_template = webpage_cst_param('Template',
	webpage_cst_radio(pagepf .. '.general.tmpl',
		{'None', 'Button', 'Directory'},
		{'', 'button', 'directory'},
		{show_page, show_page, show_page}))

local p_desc = webpage_cst_param('Description',
	webpage_cst_text(pagepf .. '.general.desc'))

local secthd_button = webpage_cst_secthd('Button Page', 'sect_button', 1)

local picfile_desc = {}
local picfile_path = {}
local picfile_dir
picfile_dir = '/apps/uiapp/rc/background'
for file in lfs.dir(picfile_dir) do
	if file ~= '.' and file ~= '..' then
		table.insert(picfile_desc, '(default) ' .. file)
		table.insert(picfile_path, 'rc/background/' .. file)
	end
end

picfile_dir = '/apps/uiapp/user/background'
local fileattr = lfs.attributes(picfile_dir)
if fileattr and fileattr.mode == 'directory' then
	for file in lfs.dir(picfile_dir) do
		if file ~= '.' and file ~= '..' then
			table.insert(picfile_desc, '(user) ' .. file)
			table.insert(picfile_path, 'user/background/' .. file)
		end
	end
end

local p_bg = webpage_cst_param('Background',
	webpage_cst_select(pagepf .. '.button.bg', picfile_desc, picfile_path))

local layout_list = {'None', 'Single Button', '2 Buttons', '3 Buttons',
	 '2x2 Buttons', '3x2 Buttons', '3x3 Buttons',
	 '3x3 Buttons Full Screen'}
local layout_value = {'', '1', '2', '3', '2x2', '3x2', '3x3', '3x3fs'}
local p_layout = webpage_cst_param('Button Layout',
	webpage_cst_select(pagepf .. '.button.layout',
		layout_list, layout_value, nil, nil, nil, nil,
		'update_buttons();show_page();'))

local p_back = webpage_cst_param('Back Button',
	webpage_cst_enabled(pagepf .. '.button.back'),
	nil, 'back_button')

local p_home = webpage_cst_param('Home Button',
	webpage_cst_enabled(pagepf .. '.button.home'),
	nil, 'home_button')

local action_list = {'None', 'Call', 'Go to Page'}
local action_value = {'', 'call', 'goto'}

local page_list = {'Dial Keypad'}
local page_name = {'dialer'}
local ndc = conf_getpn('dp.netdc.active')
for i = 1, ndc do
	table.insert(page_list, 'Door Access Keypad ' .. i)
	table.insert(page_name, 'access' .. i)
end
table.insert(page_list, 'Slideshow')
table.insert(page_list, 'Previous Page')
table.insert(page_list, 'Home Page')
table.insert(page_name, 'slideshow')
table.insert(page_name, 'back')
table.insert(page_name, 'home')

local npage = conf_getpn('ui.page.active')
for i = 1, npage do
	local desc = conf_getp('ui.page' .. i .. '.general.desc')

	if desc and desc ~= '' then
		table.insert(page_list, 'Page ' .. i .. ' - ' .. desc)
	else
		table.insert(page_list, 'Page ' .. i)
	end
	table.insert(page_name, 'page' .. i)
end

local p_outact = webpage_cst_param(
	'Action on Background Touch Event (outside of buttons)',
	webpage_cst_radio(pagepf .. '.button.action',
		action_list, action_value,
		{show_page, show_page, show_page}))

local p_out_dest = webpage_cst_param('Background Touch - Extension to Dial',
	webpage_cst_text(pagepf .. '.button.dest'),
	nil, 'page_dest', 1)

local p_out_target = webpage_cst_param('Background Touch - Target Page',
	webpage_cst_select(pagepf .. '.button.target',
		page_list, page_name),
	nil, 'page_target', 1)

local js_showall = [[<script type="text/javascript">
show_page();
</script>]]

local page_desc = conf_getp('ui.page' .. page .. '.general.desc')
local page_title
if page_desc and page_desc ~= '' then
	page_title = 'Page ' .. page .. ' - ' .. page_desc
else
	page_title = 'Page ' .. page
end

mg.write(prod_cst_header() ..
	uicommon_cst_menu(page_title) ..
	webpage_cst_mchd(flags['any'], 0) ..
	form_start .. page_start ..
	secthd_general .. webpage_subsect_hd .. js_all ..
	p_template .. p_desc ..
	webpage_subsect_ft .. webpage_sect_ft ..
	secthd_button .. webpage_subsect_hd ..
	p_bg .. p_layout .. p_back .. p_home ..
	p_outact .. p_out_dest .. p_out_target ..
	webpage_subsect_ft)

--[[
local svgtmpl_1 = '<svg width="23px" height="17px">
<rect x="0" y="0" width="23" height="17" fill="grey" />
<rect x="8" y="6" width="8" height="6" fill="black" />
</svg>'

local svgtmpl_2 = '<svg width="23px" height="17px">
<rect x="0" y="0" width="23" height="17" fill="grey" />
<rect x="2" y="6" width="8" height="6" fill="black" />
<rect x="12" y="6" width="8" height="6" fill="white" />
</svg>'

local svgtmpl_3 = '<svg width="23px" height="17px">
<rect x="0" y="0" width="23" height="17" fill="grey" />
<rect x="2" y="7" width="5" height="3" fill="black" />
<rect x="9" y="7" width="5" height="3" fill="white" />
<rect x="16" y="7" width="5" height="3" fill="white" />
</svg>'

local svgtmpl_2x2 = '<svg width="23px" height="17px">
<rect x="0" y="0" width="23" height="17" fill="grey" />
<rect x="4" y="4" width="5" height="3" fill="black" />
<rect x="13" y="4" width="5" height="3" fill="white" />
<rect x="4" y="9" width="5" height="3" fill="white" />
<rect x="13" y="9" width="5" height="3" fill="white" />
</svg>'

local svgtmpl_3x2 = '<svg width="23px" height="17px">
<rect x="0" y="0" width="23" height="17" fill="grey" />
<rect x="2" y="4" width="5" height="3" fill="black" />
<rect x="9" y="4" width="5" height="3" fill="white" />
<rect x="16" y="4" width="5" height="3" fill="white" />
<rect x="2" y="9" width="5" height="3" fill="white" />
<rect x="9" y="9" width="5" height="3" fill="white" />
<rect x="16" y="9" width="5" height="3" fill="white" />
</svg>'

local svgtmpl_3x3 = '<svg width="23px" height="17px">
<rect x="0" y="0" width="23" height="17" fill="grey" />
<rect x="2" y="2" width="5" height="3" fill="black" />
<rect x="9" y="2" width="5" height="3" fill="white" />
<rect x="16" y="2" width="5" height="3" fill="white" />
<rect x="2" y="7" width="5" height="3" fill="white" />
<rect x="9" y="7" width="5" height="3" fill="white" />
<rect x="16" y="7" width="5" height="3" fill="white" />
<rect x="2" y="12" width="5" height="3" fill="white" />
<rect x="9" y="12" width="5" height="3" fill="white" />
<rect x="16" y="12" width="5" height="3" fill="white" />
</svg>'
]]

local svgdb_2x2 = {
	{4, 4}, {13, 4},
	{4, 9}, {13, 9}}

local svgdb_3x2 = {
	{2, 4}, {9, 4}, {16, 4},
	{2, 9}, {9, 9}, {16, 9}}

local svgdb_3x3 = {
	{2, 2}, {9, 2}, {16, 2},
	{2, 7}, {9, 7}, {16, 7},
	{2, 12}, {9, 12}, {16, 12}}

local svgtmpl = {
	['hdr'] = '<svg width="23px" height="17px" class="%s" \z
		style="display:none;">\z
		<rect x="0" y="0" width="23" height="17" fill="grey" />',
	['rect'] = '<rect x="%d" y="%d" width="5" height="3" fill="%s" />',
	['ft'] = '</svg>'}

function fill_svglist(db, name)
	local list = {}
	local len = #db
	local rects
	local color
	for i = 1, len do
		rects = {}
		for j = 1, len do
			if j == i then
				color = 'black'
			else
				color = 'white'
			end
			table.insert(rects,
				string.format(svgtmpl['rect'],
					db[j][1], db[j][2], color))
		end
		table.insert(list,
			string.format(svgtmpl['hdr'], name) ..
				table.concat(rects) .. svgtmpl['ft'])
	end
	for i = len + 1, 9 do
		table.insert(list, '')
	end
	return list
end

local svglist_2x2 = fill_svglist(svgdb_2x2, 'svg2x2')
local svglist_3x2 = fill_svglist(svgdb_3x2, 'svg3x2')
local svglist_3x3 = fill_svglist(svgdb_3x3, 'svg3x3')

for i = 1, 9 do
	mg.write(webpage_cst_subsect_hd('subsect_button' .. i, 1) ..
		webpage_cst_param(svglist_2x2[i] .. svglist_3x2[i] ..
			svglist_3x3[i] .. 'Button ' .. i,
			webpage_cst_enabled(
				pagepf .. '.button' .. i .. '.use',
				{show_page, show_page})) ..
		webpage_cst_param('Text',
			webpage_cst_text(
				pagepf .. '.button' .. i .. '.label'),
			nil, 'btn' .. i .. '_label', 1) ..
		webpage_cst_param('Action',
			webpage_cst_radio(
				pagepf .. '.button' .. i .. '.action',
				action_list, action_value,
				{show_page, show_page, show_page}),
			nil, 'btn' .. i .. '_act', 1) ..
		webpage_cst_param('Action - Call Destination',
			webpage_cst_text(
				pagepf .. '.button' .. i .. '.dest'),
			nil, 'btn' .. i .. '_dest', 1) ..
		webpage_cst_param('Action - Target Page',
			webpage_cst_select(
				pagepf .. '.button' .. i .. '.target',
				page_list, page_name),
			nil, 'btn' .. i .. '_target', 1) ..
		webpage_subsect_ft)
end

local secthd_dir = webpage_cst_secthd('Directory Page', 'sect_directory', 1)

local ab_files = {}
local path = '/apps/uiapp/user/addressbook'
if lfs.attributes(path, 'mode') == 'directory' then
	for file in lfs.dir(path) do
		if file ~= '.' and file ~= '..' then
			table.insert(ab_files, file)
		end
	end
	table.sort(ab_files)
end

local p_addrbook = webpage_cst_param('Address Book File',
	webpage_cst_select(pagepf .. '.dir.addrbook', ab_files, ab_files))

mg.write(webpage_sect_ft .. secthd_dir .. webpage_subsect_hd ..
	p_addrbook ..
	webpage_subsect_ft .. webpage_sect_ft ..
	webpage_page_end .. webpage_form_end_save .. js_showall ..
	webpage_mc_ft ..
	webpage_footer)
